#!/usr/bin/env python3
# coding: UTF-8
import sys
import os
import argparse
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Draw
import openpyxl
from openpyxl.styles.alignment import Alignment


# ファイルから分子を読み込みための処理をしてイテレータを返す。
def ChemSupplier(filename):
    # 入力ファイルの存在チェック
    if os.path.exists(filename) == False:
        print("ファイルが存在しません")
        quit()

    # 拡張子を取得する
    root, ext = os.path.splitext(filename)
    ext = ext.lower()

    # SDF or SMILESの読み込み設定
    if ext == ".smi":
        suppl = Chem.SmilesMolSupplier(filename, delimiter="\t", titleLine=True)
    elif ext == ".sdf":
        suppl = Chem.SDMolSupplier(filename)
    else:
        print("不明な拡張子です")
        sys.exit()

    return suppl


# 分子に付属するプロパティのリストを取得する。
def Propaties(suppl):
    mol = next(suppl)
    propnames = mol.GetPropNames()
    suppl.reset()  # supplを先頭に戻す

    return propnames


if __name__ == "__main__":
    p = argparse.ArgumentParser(description="sdfまたはsmiファイルからエクセル(xlsx)を作成します")
    p.add_argument("input", help="input (sdf or smi)")
    p.add_argument("output", help="output (xlsx)")
    p.add_argument("-n", action="store_false", help="分子名のみ出力し、他のプロパティは出力しない")
    args = p.parse_args()

    suppl = ChemSupplier(args.input)
    propnames = Propaties(suppl)

    # excelファイル作成のためのワークブック生成
    wb = openpyxl.Workbook()
    ws = wb.worksheets[0]

    # ヘッダーを記述します。
    ws.cell(row=1, column=1).value = "structure"  # 第1列は構造式とします。
    ws.column_dimensions["A"].width = 40  # セルの幅を40にします。
    ws.cell(row=1, column=2).value = "id"  # 第2列は化合物IDとします。
    # 第3列以降にSDファイルのプロパティを入れます。
    for i, p in enumerate(propnames):
        ws.cell(row=1, column=3 + i).value = p

    for i, mol in enumerate(suppl):
        # 構造式を画像ファイルを生成します。
        mol = Chem.RemoveHs(mol)
        AllChem.Compute2DCoords(mol)
        img = Draw.MolToImage(mol, size=(320, 130))
        # セルに画像を挿入します。
        img = openpyxl.drawing.image.Image(img)
        ws.row_dimensions[i + 2].height = 100
        ws.add_image(img, ws.cell(row=i + 2, column=1).coordinate)

        # 分子のIDを挿入します。
        molname = mol.GetProp("_Name")
        ws.cell(row=i + 2, column=2).value = molname
        ws.cell(row=i + 2, column=2).alignment = Alignment(vertical="center")

        # 分子のプロパティの値をセルに挿入します。
        for j, p in enumerate(propnames):
            val = mol.GetProp(p)
            ws.cell(row=i + 2, column=3 + j).value = val
            ws.cell(row=i + 2, column=3 + j).alignment = Alignment(vertical="center")

    wb.save(args.output)  # エクセルファイルに出力
